﻿package  {
	import com.robotacid.engine.Dot;
	import com.robotacid.engine.HiddenNumber;
	import com.robotacid.engine.Line;
	import com.robotacid.engine.SavedGame;
	import com.robotacid.engine.Trig;
	import com.robotacid.engine.Util;
	import com.robotacid.engine_specific.Bullet;
	import com.robotacid.engine_specific.Gun;
	import com.robotacid.engine_specific.Missile;
	import com.robotacid.engine_specific.MissileLauncher;
	import com.robotacid.engine_specific.Debris;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Graphics;
	import flash.display.MovieClip;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	
	/**
	* ...
	* @author Aaron Steed, robotacid.com
	*/
	public class Game {
		
		public var timeline:MovieClip;
		public var canvas:Sprite;
		public var clicky:Sprite;
		public var replay:Sprite;
		public var level_clip:MovieClip;
		public var missile_holder:Sprite;
		public var debris_holder:Sprite;
		public var gfx_holder:Sprite;
		public var score_shape:Shape;
		public var debug:Graphics;
		public var bitmap:Bitmap;
		public var bitmapdata:BitmapData;
		public var path_bitmap:Bitmap;
		public var path_bitmapdata:BitmapData;
		public var missiles:Array /*Missile*/;
		public var bullets:Array /*Bullet*/;
		public var debris:Array /*Debris*/;
		public var launcher:MissileLauncher;
		public var fade:Shape;
		public var target:Missile;
		public var mouse:Dot;
		public var mouse_pressed:Boolean;
		public var score:HiddenNumber;
		public var gun:Gun;
		public var drinking:Boolean;
		public var save:SavedGame;
		
		public static const BACKGROUND_COL:uint = 0;
		public static const FOREGROUND_COL:uint = 0xFFFFFF;
		public static const BLOOD_COL:uint = 0xFF0000;
		public static const SCORE_COL:uint = 0x660000;
		
		public static const LAUNCHERS:uint = 7;
		public static const DRINK_SPEED:Number = 3.5;
		public static const WIDTH:int = 400;
		public static const HEIGHT:int = 550;
		
		public function Game(canvas:Sprite, clicky:Sprite, replay:Sprite, timeline:MovieClip) {
			this.timeline = timeline;
			this.canvas = canvas;
			this.clicky = clicky;
			this.replay = replay;
			mouse_pressed = false;
			canvas.addEventListener(Event.ENTER_FRAME, main, false, 0, true);
			clicky.addEventListener(MouseEvent.MOUSE_DOWN, mouseDown, false, 0, true);
			clicky.addEventListener(MouseEvent.MOUSE_UP, mouseUp, false, 0, true);
		}
		public function init():void {
			missiles = new Array();
			bullets = new Array();
			debris = new Array();
			missile_holder = new Sprite();
			debris_holder = new Sprite();
			level_clip = new LevelMC();
			var player_shape:Shape = new Shape();
			var charge_shape:Shape = new Shape();
			var debug_shape:Shape = new Shape();
			score_shape = new Shape();
			gfx_holder = new Sprite();
			fade = new Shape();
			fade.graphics.lineStyle(0, 0, 0);
			fade.graphics.beginFill(BACKGROUND_COL, 0.125);
			fade.graphics.drawRect(0, 0, WIDTH, HEIGHT);
			fade.graphics.endFill();
			bitmapdata = new BitmapData(WIDTH, HEIGHT, true, 0x00000000);
			bitmap = new Bitmap(bitmapdata);
			path_bitmapdata = new BitmapData(WIDTH, HEIGHT, true, 0x00000000);
			path_bitmap = new Bitmap(path_bitmapdata);
			canvas.addChild(bitmap);
			canvas.addChild(path_bitmap);
			launcher = new MissileLauncher(LAUNCHERS, this, false);
			gfx_holder.addChild(score_shape);
			gfx_holder.addChild(debris_holder);
			gfx_holder.addChild(missile_holder);
			gfx_holder.addChild(player_shape);
			gfx_holder.addChild(charge_shape);
			gfx_holder.addChild(debug_shape);
			//gfx_holder.addChild(level_clip);
			mouse = new Dot(canvas.mouseX, canvas.mouseY);
			gun = new Gun(WIDTH * 0.5, HEIGHT, player_shape, charge_shape, mouse, this);
			debug = debug_shape.graphics;
			mouse_pressed = false;
			gun.drawBody();
			score = new HiddenNumber();
			drinking = false;
			
			save = new SavedGame("test", this);
			level_clip.visible = true;
			level_clip.level.text = launcher.generation.toString();
			
		}
		public function main(e:Event):void {
			if(!replay.visible) {
				var i:int, j:int, k:int;
				target = null;
				// update missiles
				for(i = 0; i < missiles.length; i++) {
					missiles[i].main();
					//path_bitmapdata.setPixel32(missiles[i].x>>0, missiles[i].y>>0, 0x33FFFFFF);
					if(missiles[i].over) {
						target = missiles[i];
					}
					if(missiles[i].y > HEIGHT + missiles[i].radius) {
						hit(missiles[i].x);
						if(missiles[i].parent == null) launcher.breeding_pool.push(missiles[i]);
						missiles[i].mc.parent.removeChild(missiles[i].mc);
						missiles.splice(i, 1);
						i--;
					}
				}
				// update bullets
				for(i = 0; i < bullets.length; i++) {
					bullets[i].main();
					if(bullets[i].x < -bullets[i].radius || bullets[i].x > WIDTH + bullets[i].radius || bullets[i].y < -bullets[i].radius || bullets[i].y > HEIGHT + bullets[i].radius) {
						bullets[i].mc.parent.removeChild(bullets[i].mc);
						bullets.splice(i, 1);
						i--;
						continue;
					}
					for(j = 0; j < missiles.length; j++) {
						if(!missiles[j].active) continue;
						if(Util.proximity(bullets[i].x, bullets[i].y, missiles[j].x, missiles[j].y, missiles[j].radius)) {
							missiles[j].hit();
							if(missiles[j].health <= 0 || bullets[i].charged){
								var temp:MovieClip = new BangMC();
								temp.x = missiles[j].x;
								temp.y = missiles[j].y;
								for(k = 0; k < 5 + Math.random() * 5; k++) {
									addDebris(missiles[j].x, missiles[j].y);
								}
								gfx_holder.addChild(temp);
								missiles[j].active = false;
								missiles[j].mc.parent.removeChild(missiles[j].mc);
								if(missiles[j].parent == null) launcher.breeding_pool.push(missiles[j]);
								missiles.splice(j, 1);
							}
							if(bullets[i].charged) {
								for(var r:int = 0; r < 360; r += 6) {
									addBullet(bullets[i].x, bullets[i].y, Trig.cos[r], Trig.sin[r], Gun.BULLET_SPEED, false, true);
								}
							}
							bullets[i].mc.parent.removeChild(bullets[i].mc);
							bullets.splice(i, 1);
							i--;
							break;
						}
					}
				}
				// update debris
				for(i = 0; i < debris.length; i++) {
					debris[i].main();
					if(debris[i].y > HEIGHT + debris[i].radius) {
						if(debris[i].x >= gun.x - gun.width * 0.5 && debris[i].x <= gun.x + gun.width * 0.5) {
							addScore(debris[i].radius);
						}
						debris[i].mc.parent.removeChild(debris[i].mc);
						debris.splice(i, 1);
						i--;
					}
				}
				// update gun position
				var target_x = canvas.mouseX;
				target_x = Math.min(target_x, WIDTH - gun.width * 0.5);
				target_x = Math.max(target_x, gun.width * 0.5);
				gun.x += (target_x - gun.x) * 0.1;
				gun.main();
				// drink
				if(drinking) {
					score.value -= DRINK_SPEED;
					score_shape.graphics.clear();
					score_shape.graphics.lineStyle(2, SCORE_COL);
					score_shape.graphics.drawCircle(WIDTH * 0.5, HEIGHT, score.value);
					if(score.value <= 0) {
						score.value = 0;
						drinking = false;
						if(gun.health.value < Gun.MAX_HEALTH) {
							gun.health.value++;
							gun.drawBody();
						}
					}
				}
				launcher.main();
				mouse.x = canvas.mouseX;
				mouse.y = canvas.mouseY;
				gun.line.updateLine();
				render();
				if(gun.health.value <= 0) {
					mouse_pressed = false;
					replay.visible = true;
				}
				if(level_clip.visible) level_clip.visible = false;
				timeline.info.text = "bullets:" + bullets.length + "\nmissiles:" + missiles.length + "\ndebris:" + debris.length +"\nmissile clips" + missile_holder.numChildren + "\ndebris clips" + debris_holder.numChildren;
				timeline.info.text += "\nbreedpool:" + launcher.breeding_pool.length + "\nlaunchpool:" + launcher.launch_pool.length;
			} else {
				if(mouse_pressed) {
					replay.visible = false;
					init();
				}
			}
			
		}
		public function addDebris(x:Number, y:Number):void {
			var temp:Shape = new Shape();
			temp.x = x;
			temp.y = y;
			debris_holder.addChild(temp);
			debris.push(new Debris(temp));
		}
		public function hit(x:Number):void {
			gun.hit(x);
			if(gun.health.value <= 0) {
			}
		}
		public function addScore(n:Number):void {
			score.value += n;
			// score
			drawScore();
			if(score.value > HEIGHT) drinking = true;
		}
		public function drawScore():void {
			score_shape.graphics.clear();
			score_shape.graphics.lineStyle(2, SCORE_COL);
			score_shape.graphics.drawCircle(WIDTH * 0.5, HEIGHT, score.value);
		}
		public function render():void {
			bitmapdata.draw(fade);
			bitmapdata.draw(gfx_holder);
		}
		public function addMissile(missile:Missile):void {
			missiles.push(missile);
			missile_holder.addChild(missile.mc);
		}
		public function mouseDown(e:MouseEvent):void {
			mouse_pressed = true;
		}
		public function mouseUp(e:MouseEvent):void {
			mouse_pressed = false;
		}
		public function addBullet(x:Number, y:Number, dx:Number, dy:Number, speed:Number, charged:Boolean = false, burst:Boolean = false):void {
			var temp:Shape = new Shape();
			missile_holder.addChild(temp);
			var temp_bullet:Bullet = new Bullet(temp, x, y, dx, dy, speed, this, charged);
			if(target != null && !burst) {
				temp_bullet.track = new Line(temp_bullet, target);
				if(target.evasive) {
					target.addVelocity(0, Missile.MAX_SPEED);
				}
			}
			bullets.push(temp_bullet);
		}
		
	}
	
}