﻿package com.robotacid.engine {
	import flash.display.Graphics;
	
	/**
	* Line
	* @author Aaron Steed
	* 
	* Line class for calculation of positions and directions
	*/
	public class Line {
		
		public var a:Dot;// 		Old position x,y
		public var b:Dot;// 		Current position x,y
		public var vx:Number;// 	distance between p0.x and p1.x
		public var vy:Number;// 	distance between p0.y and p1.y
		public var len:Number;// 	vector length (pythagoras length of p0 to p1)
		public var dx:Number;// 	x value of unit vector (equal to Math.cos(theta))
		public var dy:Number;// 	y value of unit vector (equal to Math.sin(theta))
		public var rx:Number;// 	x value of right hand unit vector
		public var ry:Number;// 	y value of right hand unit vector
		public var lx:Number;// 	x value of left hand unit vector
		public var ly:Number;// 	y value of left hand unit vector
		public var theta:Number;// 	rotation - updated by calling atan2 or rotate methods
		
		public function Line(a:Dot, b:Dot){
			this.a = a;
			this.b = b;
			updateLine();
		}
		// Initialise Line
		public function updateLine():void{
			vx = b.x - a.x;
			vy = b.y - a.y;
			// length of vector
			len = Math.sqrt(vx * vx + vy * vy);
			// normalized unit-sized components
			if (len > 0) {
				dx = vx / len;
				dy = vy / len;
			} else {
				dx = 0.0;
				dy = 0.0;
			}
			// right hand normal
			rx = -dy;
			ry = dx;
			// left hand normal
			lx = dy;
			ly = -dx;
		}
		// Rotate vector around point A
		public function rotateA(angle:Number):void{
			angle += atan2();
			theta = angle;
			b.x = a.x + Math.cos(angle) * len;
			b.y = a.y + Math.sin(angle) * len;
			updateLine();
		}
		// Rotate vector around point B
		public function rotateB(angle:Number):void{
			angle += atan2();
			theta = angle;
			a.x = b.x + Math.cos(angle) * len;
			a.y = b.y + Math.sin(angle) * len;
			updateLine();
		}
		// Set theta and return it's value
		public function atan2():Number{
			theta = Math.atan2(vy, vx);
			return theta;
		}
		// Return a point along the line
		public function lerp(n:Number):Object{
			return {x:b.x+((a.x-b.x) * n), y:b.y+((a.y-b.y) * n)};
		}
		// return the a and b properties of this line as a string
		public function toString():String{
			return "a:("+a.x+","+a.y+") b:("+b.x+","+b.y+")";
		}
		// draw this line
		public function draw(gfx:Graphics):void{
			gfx.moveTo(a.x, a.y);
			gfx.lineTo(b.x, b.y);
		}
		// Return a copy of this line with new Dot objects
		public function copy():Line{
			return new Line(new Dot(a.x, a.y), new Dot(b.x, b.y));
		}
		
	}
	
}