﻿package com.robotacid.engine_specific {
	import com.robotacid.engine.Dot;
	import flash.display.Shape;
	
	/**
	* ...
	* @author Aaron Steed
	*/
	public class MissileLauncher {
		
		public var spawn_points:Array;
		public var count:int;
		public var g:Game;
		public var launch_pool:Array /*Missile*/;
		public var breeding_pool:Array /*Missile*/;
		
		public var cross_over_rate:Number = 0.7;
		public var mutation_rate:Number = 0.001;
		public var generation:int = 0;
		public var active:Boolean;
		
		public static const SPAWN_RATE:Number = 0.2;
		public static const MAX_SUBROUTINES:int = 4;
		public static const PROGRAM_DEPTH:int = 5;
		
		public static const DELAY:int = 10;
		public static const POOL_SIZE:int = 7;
		
		public function MissileLauncher(n:int, g:Game, active:Boolean) {
			this.g = g;
			spawn_points = new Array();
			var step:Number = Game.WIDTH / (n+1);
			for(var i:int = 1; i < n+1; i++) {
				spawn_points.push(new Dot(i * step, -(Missile.RADIUS + Missile.HEALTH)));
			}
			launch_pool = new Array();
			for(i = 0; i < POOL_SIZE; i++) {
				launch_pool.push(createMissile(i));
			}
			breeding_pool = new Array();
			count = DELAY;
			this.active = active;
		}
		public function main():void {
			if(launch_pool.length > 0) {
				g.addMissile(launch_pool.shift());
			} else if(breeding_pool.length >= POOL_SIZE) {
				generation++;
				//g.level_clip.visible = true;
				g.level_clip.level.text = generation.toString();
				active = true;
				breeding_pool.sortOn("fitness", Array.DESCENDING);
				while(breeding_pool.length > 1) {
					var a:Missile = breeding_pool.shift();
					a.age++;
					a.reset();
					launch_pool.push(a);
				}
				while(breeding_pool.length > 0) {
					breeding_pool.pop();
					launch_pool.push(createMissile(Math.random() * POOL_SIZE));
				}
				g.save.missiles = new Array();
				for(var i:int = 0; i < launch_pool.length; i++) {
					g.save.missiles.push(launch_pool[i].save());
				}
				if(g.gun.health.value <= 0) g.gun.health.value = 1;
				g.save.save();
			}
		}
		public function crossOver(a:Array, b:Array):void {
			if(Math.random() > cross_over_rate) return;
			var position:int = Math.random() * a.length;
			for(var i:int = 0; i < a.length - 1; i++) {
				var temp:* = a[i];
				a[i] = b[i];
				b[i] = temp;
			}
		}
		public function createProgram():Object {
			var routines:Array = new Array();
			var subs:int = 0;
			var strings:Array = new Array();
			for(var j:int = 0; j < MAX_SUBROUTINES; j++) {
				var list:Array = new Array();
				var string:String = "";
				for(var i:int = 0; i < 20; i++) {
					// add movement
					var r:int, f:int, w:int;
					r = (Math.random() * 18) * (Math.random() * 10);
					r *= Math.random() > 0.5 ? 1 : -1;
					r += 180;
					r = (r + 270) % 360;
					f = 1 + ((Math.random() * Missile.MAX_SPEED) >> 0);
					w = (Math.random() * 100) >> 0;
					list.push("R", r, "F", f, "W", w);
					string += "R " + r + " F " + f + " W " + w + " ";
					// add spawn command
					if(Math.random() < SPAWN_RATE) {
						var index:int = (Math.random() * MAX_SUBROUTINES) >> 0
						list.push("S", index);
						string += "S " + index + " ";
					}
				}
				// trim the string
				string = string.replace(/^[ \t]+|[ \t]+$/g, "");
				strings.push(string);
				// add the list
				routines.push(list);
			}
			return {list:routines, strings:strings};
		}
		public function createMissile(place:int):Missile {
			var r:int = Math.random() *  spawn_points.length;
			var temp:Shape = new Shape();
			temp.x = spawn_points[place].x;
			temp.y = spawn_points[place].y;
			var temp_program:Object = createProgram();
			return new Missile(Missile.HEALTH, temp, temp_program.list[0], temp_program.list, temp_program.strings, g);
		}
		public function spawnMissile(health:int, missile:Missile, program:Array, subroutines:Array):void {
			var temp:Shape = new Shape();
			temp.x = missile.x;
			temp.y = missile.y;
			g.addMissile(new Missile(health, temp, program, subroutines, [], g, missile));
		}
		public function decodeSavedMissiles(str:String):void {
			var i:int, j:int;
			var missile_strings:Array = str.split("|");
			var list:Array = new Array();
			for(i = 0; i < missile_strings.length; i++) {
				var stuff:Array = missile_strings[i].split(":");
				list.push( { strings:new Array(stuff[3], stuff[4], stuff[5], stuff[6]), age:int(stuff[0]), ix:Number(stuff[1]), iy:Number(stuff[2]) } );
			}
			launch_pool = new Array();
			for(i = 0; i < list.length; i++) {
				var routines:Array = new Array();
				for(j = 0; j < list[i].strings.length; j++) {
					routines.push(createCommandList(list[i].strings[j]));
				}
				var temp:Shape = new Shape();
				temp.x = list[i].ix;
				temp.y = list[i].iy;
				var missile:Missile = new Missile(Missile.HEALTH, temp, routines[0], routines, list[i].strings, g);
				missile.age = list[i].age;
				missile.reset();
				launch_pool.push(missile);
			}
		}
		public function createCommandList(path_string:String):Array{
			// generate path array
			var path:Array = path_string.split(" ");
			// convert string numbers into integers
			var n:int;
			for(var k:int = 0; k < path.length; k++){
				n = parseInt(path[k]);
				if(isFinite(path[k])){
					path[k] = n;
				}
			}
			return path;
		}
		
	}
	
}